syntax = "proto3";

package xray.transport;
option csharp_namespace = "Xray.Transport.Global";
option go_package = "github.com/xtls/xray-core/transport/global";
option java_package = "com.xray.transport.global";
option java_multiple_files = true;

import "transport/internet/config.proto";

// Global transport settings. This affects all type of connections that go
// through Xray. Deprecated. Use each settings in StreamConfig.
message Config {
  option deprecated = true;
  repeated xray.transport.internet.TransportConfig transport_settings = 1;
}
