/*
Package websocket implements WebSocket transport

WebSocket transport implements an HTTP(S) compliable, surveillance proof transport method with plausible deniability.
*/
package websocket

//go:generate go run github.com/xtls/xray-core/common/errors/errorgen
